@@ -9,7 +9,8 @@
 #include "./encode.h"
 
 #include <algorithm>
-#include <cstring>
+#include <cstdlib>  /* free, malloc */
+#include <cstring>  /* memset */
 #include <limits>
 
 #include "./backward_references.h"
@@ -193,6 +194,7 @@ BrotliCompressor::BrotliCompressor(BrotliParams params)
       storage_size_(0),
       storage_(0),
       large_table_(NULL),
+      cmd_code_numbits_(0),
       command_buf_(NULL),
       literal_buf_(NULL) {
   // Sanitize params.
@@ -216,9 +218,6 @@ BrotliCompressor::BrotliCompressor(BrotliParams params)
                                std::max(kMinInputBlockBits, params_.lgblock));
   }
 
-  // Set maximum distance, see section 9.1. of the spec.
-  max_backward_distance_ = (1 << params_.lgwin) - 16;
-
   // Initialize input and literal cost ring buffers.
   // We allocate at least lgwin + 1 bits for the ring buffer so that the newly
   // added block fits there completely and we still get lgwin bits and at least
@@ -251,11 +250,11 @@ BrotliCompressor::BrotliCompressor(BrotliParams params)
   }
 
   // Initialize hashers.
-  hash_type_ = std::min(9, params_.quality);
+  hash_type_ = std::min(10, params_.quality);
   hashers_->Init(hash_type_);
 }
 
-BrotliCompressor::~BrotliCompressor() {
+BrotliCompressor::~BrotliCompressor(void) {
   delete[] storage_;
   free(commands_);
   delete ringbuffer_;
@@ -327,7 +326,7 @@ void BrotliCompressor::BrotliSetCustomDictionary(
   if (size > 1) {
     prev_byte2_ = dict[size - 2];
   }
-  hashers_->PrependCustomDictionary(hash_type_, size, dict);
+  hashers_->PrependCustomDictionary(hash_type_, params_.lgwin, size, dict);
 }
 
 bool BrotliCompressor::WriteBrotliData(const bool is_last,
@@ -393,8 +392,8 @@ bool BrotliCompressor::WriteBrotliData(const bool is_last,
 
   CreateBackwardReferences(bytes, WrapPosition(last_processed_pos_),
                            is_last, data, mask,
-                           max_backward_distance_,
                            params_.quality,
+                           params_.lgwin,
                            hashers_,
                            hash_type_,
                            dist_cache_,
@@ -814,7 +813,7 @@ class BrotliBlockReader {
  public:
   explicit BrotliBlockReader(size_t block_size)
       : block_size_(block_size), buf_(NULL) {}
-  ~BrotliBlockReader() { delete[] buf_; }
+  ~BrotliBlockReader(void) { delete[] buf_; }
 
   const uint8_t* Read(BrotliIn* in, size_t* bytes_read, bool* is_last) {
     *bytes_read = 0;
