@@ -1,17 +1,9 @@
-// Copyright 2009 Google Inc. All Rights Reserved.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
+/* Copyright 2009 Google Inc. All Rights Reserved.
+
+   Distributed under MIT license.
+   See file LICENSE for detail or copy at https://opensource.org/licenses/MIT
+*/
+
 // Convience routines to make Brotli I/O classes from some memory containers and
 // files.
 
@@ -92,7 +84,7 @@ BrotliFileIn::BrotliFileIn(FILE* f, size_t max_read_size)
       buf_(new char[max_read_size]),
       buf_size_(max_read_size) { }
 
-BrotliFileIn::~BrotliFileIn() {
+BrotliFileIn::~BrotliFileIn(void) {
   delete[] buf_;
 }
 
